@@ -1,4 +1,5 @@
 #ifndef CONFIG_H
+#define CONFIG_H
 /* ============================================================
  * Platform options
  * ============================================================ */
@@ -60,4 +61,9 @@
 
 #define UNUSED(A) (void)(A)
 
+/* Android Bionic libpthread implementation doesn't have pthread_cancel */
+#ifndef ANDROID
+#  define HAVE_PTHREAD_CANCEL
+#endif
+
 #endif
