@@ -96,16 +96,17 @@ tests = [
 
     (2, './06-bridge-b2br-disconnect-qos1.py'),
     (2, './06-bridge-b2br-disconnect-qos2.py'),
+    (2, './06-bridge-b2br-late-connection-retain.py'),
+    (2, './06-bridge-b2br-late-connection.py'),
     (2, './06-bridge-b2br-remapping.py'),
     (2, './06-bridge-br2b-disconnect-qos1.py'),
     (2, './06-bridge-br2b-disconnect-qos2.py'),
     (2, './06-bridge-br2b-remapping.py'),
     (2, './06-bridge-fail-persist-resend-qos1.py'),
     (2, './06-bridge-fail-persist-resend-qos2.py'),
-    (2, './06-bridge-reconnect-local-out.py'),
+    (1, './06-bridge-no-local.py'),
     (3, './06-bridge-per-listener-settings.py'),
-    (2, './06-bridge-b2br-late-connection.py'),
-    (2, './06-bridge-b2br-late-connection-retain.py'),
+    (2, './06-bridge-reconnect-local-out.py'),
 
     (1, './07-will-delay-reconnect.py'),
     (1, './07-will-delay-recover.py'),
@@ -117,6 +118,7 @@ tests = [
     (1, './07-will-null.py'),
     (1, './07-will-properties.py'),
     (1, './07-will-qos0.py'),
+    (1, './07-will-reconnect-1273.py'),
 
     (2, './08-ssl-bridge.py'),
     (2, './08-ssl-connect-cert-auth-crl.py'),
